"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.process_instruction = exports.loadINST = exports.jumpINST = exports.zeroEX = exports.signEX = exports.setTARGET = exports.TARGET = exports.IOFFSET = exports.setIMM = exports.IMM = exports.setSHAMT = exports.SHAMT = exports.setRD = exports.RD = exports.setRT = exports.RT = exports.setRS = exports.RS = exports.setFUNC = exports.FUNC = exports.setOPCODE = exports.OPCODE = void 0;
var constants_1 = require("@utils/constants");
var functions_1 = require("@utils/functions");
function OPCODE(INST) {
    return INST.opcode;
}
exports.OPCODE = OPCODE;
function setOPCODE(INST, VAL) {
    INST.opcode = VAL;
}
exports.setOPCODE = setOPCODE;
function FUNC(INST) {
    return INST.funcCode;
}
exports.FUNC = FUNC;
function setFUNC(INST, VAL) {
    INST.funcCode = VAL;
}
exports.setFUNC = setFUNC;
function RS(INST) {
    return INST.rs;
}
exports.RS = RS;
function setRS(INST, VAL) {
    INST.rs = VAL;
}
exports.setRS = setRS;
function RT(INST) {
    return INST.rt;
}
exports.RT = RT;
function setRT(INST, VAL) {
    INST.rt = VAL;
}
exports.setRT = setRT;
function RD(INST) {
    return INST.rd;
}
exports.RD = RD;
function setRD(INST, VAL) {
    INST.rd = VAL;
}
exports.setRD = setRD;
function SHAMT(INST) {
    return INST.shamt;
}
exports.SHAMT = SHAMT;
function setSHAMT(INST, VAL) {
    INST.shamt = VAL;
}
exports.setSHAMT = setSHAMT;
function IMM(INST) {
    return INST.imm;
}
exports.IMM = IMM;
function setIMM(INST, VAL) {
    INST.imm = VAL;
}
exports.setIMM = setIMM;
function IOFFSET(INST) {
    return IMM(INST);
}
exports.IOFFSET = IOFFSET;
function TARGET(INST) {
    return INST.target;
}
exports.TARGET = TARGET;
function setTARGET(INST, VAL) {
    INST.target = VAL;
}
exports.setTARGET = setTARGET;
function signEX(X) {
    if (X & 0x8000)
        return X | 0xffff0000;
    else
        return X;
}
exports.signEX = signEX;
function zeroEX(X) {
    return X & 0x0000ffff;
}
exports.zeroEX = zeroEX;
function jumpINST(TARGET) {
    constants_1.currentState.PC = TARGET;
}
exports.jumpINST = jumpINST;
function loadINST(LD, MASK) {
    return LD & MASK;
}
exports.loadINST = loadINST;
function process_instruction() {
    /*
     * Procedure: process_instruction
     * Purpose: Process one instruction
     */
    var info = (0, functions_1.getInstInfo)(constants_1.currentState.PC);
    // R type
    if (OPCODE(info) === 0x0) {
        var rs = RS(info);
        var rt = RT(info);
        var rd = RD(info);
        var shamt = SHAMT(info);
        var funcCode = FUNC(info);
        // ADD
        if (funcCode === 32) {
            constants_1.currentState.REGS[rd] = constants_1.currentState.REGS[rs] + constants_1.currentState.REGS[rt];
        }
        // ADDU
        else if (funcCode === 33) {
            constants_1.currentState.REGS[rd] = constants_1.currentState.REGS[rs] + constants_1.currentState.REGS[rt];
        }
        // SUB
        else if (funcCode === 34) {
            constants_1.currentState.REGS[rd] = constants_1.currentState.REGS[rs] - constants_1.currentState.REGS[rt];
        }
        // SUBU
        else if (funcCode === 35) {
            constants_1.currentState.REGS[rd] = constants_1.currentState.REGS[rs] - constants_1.currentState.REGS[rt];
        }
        // AND
        else if (funcCode === 36) {
            constants_1.currentState.REGS[rd] = constants_1.currentState.REGS[rs] & constants_1.currentState.REGS[rt];
        }
        // OR
        else if (funcCode === 37) {
            constants_1.currentState.REGS[rd] = constants_1.currentState.REGS[rs] | constants_1.currentState.REGS[rt];
        }
        // NOR
        else if (funcCode === 39) {
            constants_1.currentState.REGS[rd] = ~(constants_1.currentState.REGS[rs] | constants_1.currentState.REGS[rt]);
        }
        // SLT
        else if (funcCode === 42) {
            if (constants_1.currentState.REGS[rs] < constants_1.currentState.REGS[rt])
                constants_1.currentState.REGS[rd] = 1;
            else
                constants_1.currentState.REGS[rd] = 0;
        }
        // SLTU
        else if (funcCode === 43) {
            if (constants_1.currentState.REGS[rs] < constants_1.currentState.REGS[rt])
                constants_1.currentState.REGS[rd] = 1;
            else
                constants_1.currentState.REGS[rd] = 0;
        }
        // SLL
        else if (funcCode === 0) {
            constants_1.currentState.REGS[rd] = constants_1.currentState.REGS[rt] << shamt;
        }
        // SRL
        else if (funcCode === 2) {
            constants_1.currentState.REGS[rd] = constants_1.currentState.REGS[rt] >> shamt;
        }
        // JR
        else if (funcCode === 8) {
            constants_1.currentState.PC = constants_1.currentState.REGS[rs];
        }
        if (funcCode !== 8)
            constants_1.currentState.PC += constants_1.BYTES_PER_WORD;
    }
    // J
    else if (OPCODE(info) === 0x2) {
        var target = TARGET(info) << 2;
        jumpINST(target);
    }
    // JAL
    else if (OPCODE(info) === 0x3) {
        var target = TARGET(info) << 2;
        constants_1.currentState.REGS[31] = constants_1.currentState.PC + 8;
        jumpINST(target);
    }
    // I type
    else {
        var rs = RS(info);
        var rt = RT(info);
        var imm = IMM(info);
        var opcode = OPCODE(info);
        // ADDI
        if (opcode === 0x8) {
            constants_1.currentState.REGS[rt] = constants_1.currentState.REGS[rs] + signEX(imm);
        }
        // ADDIU
        else if (opcode === 0x9) {
            constants_1.currentState.REGS[rt] = constants_1.currentState.REGS[rs] + signEX(imm);
        }
        // ANDI
        else if (opcode === 0xc) {
            constants_1.currentState.REGS[rt] = constants_1.currentState.REGS[rs] & zeroEX(imm);
        }
        // BEQ
        else if (opcode === 0x4) {
            if (constants_1.currentState.REGS[rs] === constants_1.currentState.REGS[rt])
                constants_1.currentState.PC += imm * 4;
        }
        // BNE
        else if (opcode === 0x5) {
            if (constants_1.currentState.REGS[rs] !== constants_1.currentState.REGS[rt]) {
                constants_1.currentState.PC += signEX(imm) * 4;
            }
        }
        // LHU
        else if (opcode === 0x25) {
            constants_1.currentState.REGS[rt] = loadINST((0, functions_1.memRead)(constants_1.currentState.REGS[rs] + signEX(IOFFSET(info))), 0x0000ffff);
        }
        // LUI
        else if (opcode === 0xf) {
            constants_1.currentState.REGS[rt] = imm << 16;
        }
        // LW
        else if (opcode == 0x23) {
            constants_1.currentState.REGS[rt] = (0, functions_1.memRead)(constants_1.currentState.REGS[rs] + signEX(IOFFSET(info)));
        }
        // ORI
        else if (opcode === 0xd) {
            constants_1.currentState.REGS[rt] = constants_1.currentState.REGS[rs] | zeroEX(imm);
        }
        // SLTI
        else if (opcode === 0xa) {
            if (constants_1.currentState.REGS[rs] < signEX(imm))
                constants_1.currentState.REGS[rt] = 1;
            else
                constants_1.currentState.REGS[rt] = 0;
        }
        // SLTIU
        else if (opcode === 0xb) {
            if (constants_1.currentState.REGS[rs] < signEX(imm))
                constants_1.currentState.REGS[rt] = 1;
            else
                constants_1.currentState.REGS[rt] = 0;
        }
        // SH
        else if (opcode === 0x29) {
            (0, functions_1.memWriteHalf)(constants_1.currentState.REGS[rs] + signEX(IOFFSET(info)), constants_1.currentState.REGS[rt]);
        }
        // SW
        else if (opcode === 0x2b) {
            (0, functions_1.memWrite)(constants_1.currentState.REGS[rs] + signEX(IOFFSET(info)), constants_1.currentState.REGS[rt]);
        }
        constants_1.currentState.PC += constants_1.BYTES_PER_WORD;
    }
    if (constants_1.currentState.PC - constants_1.MEM_TEXT_START === constants_1.NUM_INST * 4) {
        (0, constants_1.changeRunBit)();
    }
}
exports.process_instruction = process_instruction;
