export interface Iinstruction {
    opcode: number;
    funcCode: number;
    value: number;
    target: number;
    rs: number;
    rt: number;
    imm: number;
    rd: number;
    shamt: number;
    encoding: number;
    expr: number;
}
interface IBcolors {
    readonly BLUE: string;
    readonly YELLOW: string;
    readonly GREEN: string;
    readonly RED: string;
    readonly ENDC: string;
}
interface ISection {
    readonly DATA: number;
    readonly TEXT: number;
    readonly MAX_SIZE: number;
}
export interface SymbolTableType {
    readonly name: string;
    readonly address: number;
}
export interface IinstList {
    readonly [key: string]: instT;
}
export interface ISYMBOL_TABLE {
    [key: string]: number;
}
export declare const DEBUG = 0;
export declare const MAX_SYMBOL_TABLE_SIZE = 1024;
export declare const MEM_TEXT_START = 4194304;
export declare const MEM_TEXT_SIZE = 1048576;
export declare const MEM_DATA_START = 268435456;
export declare const MEM_DATA_SIZE = 1048576;
export declare const MEM_STACK_START = 2147483648;
export declare const MEM_STACK_SIZE = 1048576;
export declare const BYTES_PER_WORD = 4;
export declare const INST_LIST_LEN = 27;
export declare const MIPS_REGS = 32;
export declare const MEM_GROW_UP = 1;
export declare const MEM_GROW_DOWN = -1;
export declare const MEM_NREGIONS = 3;
export declare const DEBUG_SET = 0;
export declare const MEM_DUMP_SET = 1;
export declare let RUN_BIT: number;
export declare let INSTRUCTION_COUNT: number;
export declare let memText: memRegionT;
export declare let memData: memRegionT;
export declare let memStack: memRegionT;
export declare let memRegions: memRegionT[];
export declare let currentState: cpuState;
export declare let NUM_INST: number;
export declare const INST_INFO: Iinstruction[];
export declare const section: ISection;
export declare const bcolors: IBcolors;
export declare const pType: string[];
export declare class instT {
    name: string;
    type: string;
    op: string;
    funct: string;
    constructor(name: string, op: string, type: string, funct: string);
}
export declare class symbolT {
    name: string;
    address: number;
    constructor();
}
export declare class laStruct {
    op: string;
    rt: string;
    imm: string;
    constructor(op: string, rt: string, imm: string);
}
export declare class cpuState {
    PC: number;
    REGS: number[];
    constructor();
}
export declare class instruction {
    opcode: number;
    funcCode: number;
    value: number;
    target: number;
    rs: number;
    rt: number;
    imm: number;
    rd: number;
    shamt: number;
    encoding: number;
    expr: number;
    constructor();
}
export declare function initialize(binary: string[], textSize: number, dataSize: number): Iinstruction[];
export declare class memRegionT {
    start: number;
    size: number;
    mem: number[];
    offBound: number;
    type: number;
    dirty: boolean;
    constructor(start: number, size: number, type?: number);
}
export declare const instList: IinstList;
export declare const symbolStruct: symbolT;
export declare let SYMBOL_TABLE: ISYMBOL_TABLE;
export declare const resetSymbolTable: () => void;
export declare const initializeMem: () => void;
export declare const instAddOne: () => void;
export declare const numInstSub: () => void;
export declare const changeRunBit: () => void;
export {};
