"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.changeRunBit = exports.numInstSub = exports.instAddOne = exports.initializeMem = exports.resetSymbolTable = exports.SYMBOL_TABLE = exports.symbolStruct = exports.instList = exports.memRegionT = exports.initialize = exports.instruction = exports.cpuState = exports.laStruct = exports.symbolT = exports.instT = exports.pType = exports.bcolors = exports.section = exports.INST_INFO = exports.NUM_INST = exports.currentState = exports.memRegions = exports.memStack = exports.memData = exports.memText = exports.INSTRUCTION_COUNT = exports.RUN_BIT = exports.MEM_DUMP_SET = exports.DEBUG_SET = exports.MEM_NREGIONS = exports.MEM_GROW_DOWN = exports.MEM_GROW_UP = exports.MIPS_REGS = exports.INST_LIST_LEN = exports.BYTES_PER_WORD = exports.MEM_STACK_SIZE = exports.MEM_STACK_START = exports.MEM_DATA_SIZE = exports.MEM_DATA_START = exports.MEM_TEXT_SIZE = exports.MEM_TEXT_START = exports.MAX_SYMBOL_TABLE_SIZE = exports.DEBUG = void 0;
var functions_1 = require("@utils/functions");
exports.DEBUG = 0;
exports.MAX_SYMBOL_TABLE_SIZE = 1024;
exports.MEM_TEXT_START = 0x00400000;
exports.MEM_TEXT_SIZE = 0x00100000;
exports.MEM_DATA_START = 0x10000000;
exports.MEM_DATA_SIZE = 0x00100000;
exports.MEM_STACK_START = 0x80000000;
exports.MEM_STACK_SIZE = 0x00100000;
exports.BYTES_PER_WORD = 4;
exports.INST_LIST_LEN = 27;
exports.MIPS_REGS = 32;
exports.MEM_GROW_UP = 1;
exports.MEM_GROW_DOWN = -1;
exports.MEM_NREGIONS = 3;
exports.DEBUG_SET = 0;
exports.MEM_DUMP_SET = 1;
exports.RUN_BIT = 1;
exports.INSTRUCTION_COUNT = 0;
exports.INST_INFO = [];
exports.section = {
    DATA: 0,
    TEXT: 1,
    MAX_SIZE: 2,
};
exports.bcolors = {
    BLUE: '\x1B[34m',
    YELLOW: '\x1B[33m',
    GREEN: '\x1B[32m',
    RED: '\x1B[31m',
    ENDC: '\x1B[0m',
};
var start = "[".concat(exports.bcolors.BLUE, "START").concat(exports.bcolors.ENDC, "]  ");
var done = "[".concat(exports.bcolors.YELLOW, "DONE").concat(exports.bcolors.ENDC, "]  ");
var success = "[".concat(exports.bcolors.GREEN, "SUCCESS").concat(exports.bcolors.ENDC, "]  ");
var error = "[".concat(exports.bcolors.RED, "ERROR").concat(exports.bcolors.ENDC, "]  ");
exports.pType = [start, done, success, error];
// Structure Declaration
var instT = /** @class */ (function () {
    function instT(name, op, type, funct) {
        this.name = name;
        this.op = op;
        this.type = type;
        this.funct = funct;
    }
    return instT;
}());
exports.instT = instT;
var symbolT = /** @class */ (function () {
    function symbolT() {
        this.name = '';
        this.address = 0;
    }
    return symbolT;
}());
exports.symbolT = symbolT;
var laStruct = /** @class */ (function () {
    function laStruct(op, rt, imm) {
        this.op = op;
        this.rt = rt;
        this.imm = imm;
    }
    return laStruct;
}());
exports.laStruct = laStruct;
var cpuState = /** @class */ (function () {
    function cpuState() {
        this.PC = 0;
        this.REGS = Array.from({ length: 32 }, function () { return 0; });
        this.REGS[29] = exports.MEM_STACK_START;
    }
    return cpuState;
}());
exports.cpuState = cpuState;
var instruction = /** @class */ (function () {
    function instruction() {
        this.opcode = 0;
        this.funcCode = 0;
        this.value = 0;
        this.target = 0;
        this.rs = 0;
        this.rt = 0;
        this.imm = 0;
        this.rd = 0;
        this.shamt = 0;
        this.encoding = 0;
        this.expr = 0;
    }
    return instruction;
}());
exports.instruction = instruction;
function initialize(binary, textSize, dataSize) {
    (0, functions_1.initMemory)();
    // Load program and service routines into mem
    var textIndex = 0;
    var size = 0;
    exports.NUM_INST = ~~(textSize / 4);
    // initial memory allocation of text segment
    for (var i = 0; i < exports.NUM_INST; i++)
        exports.INST_INFO.push(new instruction());
    (0, functions_1.initInstInfo)(exports.NUM_INST, exports.INST_INFO);
    binary.forEach(function (buffer) {
        if (size < textSize) {
            exports.INST_INFO[textIndex] = (0, functions_1.parseInstr)(buffer, size);
            textIndex += 1;
        }
        else if (size < textSize + dataSize) {
            (0, functions_1.parseData)(buffer, size - textSize);
        }
        size += 4;
    });
    exports.currentState.PC = exports.MEM_TEXT_START;
    exports.RUN_BIT = 1;
    return exports.INST_INFO;
}
exports.initialize = initialize;
/*
  All simulated memory will be managed by this class
  use the mem_write and mem_read functions to
  access/modify the simulated memory
*/
var memRegionT = /** @class */ (function () {
    function memRegionT(start, size, type) {
        if (type === void 0) { type = exports.MEM_GROW_UP; }
        this.start = start;
        this.size = size;
        this.mem = [];
        this.offBound = -(size - 4) * type;
        this.type = type;
        this.dirty = false;
    }
    return memRegionT;
}());
exports.memRegionT = memRegionT;
// Global Variable Declaration
var SLL = new instT('sll', '000000', 'R', '000000');
var SRL = new instT('srl', '000000', 'R', '000010');
var JR = new instT('jr', '000000', 'R', '001000');
var ADD = new instT('add', '000000', 'R', '100000');
var ADDU = new instT('addu', '000000', 'R', '100001');
var AND = new instT('and', '000000', 'R', '100100');
var NOR = new instT('nor', '000000', 'R', '100111');
var OR = new instT('or', '000000', 'R', '100101');
var SLT = new instT('slt', '000000', 'R', '101010');
var SLTU = new instT('sltu', '000000', 'R', '101011');
var SUB = new instT('sub', '000000', 'R', '100010');
var SUBU = new instT('subu', '000000', 'R', '100011');
var LUI = new instT('lui', '001111', 'I', '');
var BEQ = new instT('beq', '000100', 'I', '');
var BNE = new instT('bne', '000101', 'I', '');
var LW = new instT('lw', '100011', 'I', '');
var LHU = new instT('lhu', '100101', 'I', '');
var SW = new instT('sw', '101011', 'I', '');
var SH = new instT('sh', '101001', 'I', '');
var ADDI = new instT('addi', '001000', 'I', '');
var ADDIU = new instT('addiu', '001001', 'I', '');
var ANDI = new instT('andi', '001100', 'I', '');
var ORI = new instT('ori', '001101', 'I', '');
var SLTI = new instT('slti', '001010', 'I', '');
var SLTIU = new instT('sltiu', '001011', 'I', '');
var J = new instT('j', '000010', 'J', '');
var JAL = new instT('jal', '000011', 'J', '');
exports.instList = {
    add: ADD,
    addi: ADDI,
    addiu: ADDIU,
    addu: ADDU,
    and: AND,
    andi: ANDI,
    beq: BEQ,
    bne: BNE,
    j: J,
    jal: JAL,
    jr: JR,
    lhu: LHU,
    lui: LUI,
    lw: LW,
    nor: NOR,
    or: OR,
    ori: ORI,
    slt: SLT,
    slti: SLTI,
    sltiu: SLTIU,
    sltu: SLTU,
    sll: SLL,
    srl: SRL,
    sh: SH,
    sw: SW,
    sub: SUB,
    subu: SUBU,
};
// Global symbol table
exports.symbolStruct = new symbolT();
exports.SYMBOL_TABLE = {};
var resetSymbolTable = function () {
    exports.SYMBOL_TABLE = {};
};
exports.resetSymbolTable = resetSymbolTable;
var initializeMem = function () {
    exports.INSTRUCTION_COUNT = 0;
    exports.RUN_BIT = 1;
    exports.memText = new memRegionT(exports.MEM_TEXT_START, exports.MEM_TEXT_SIZE);
    exports.memData = new memRegionT(exports.MEM_DATA_START, exports.MEM_DATA_SIZE);
    exports.memStack = new memRegionT(exports.MEM_STACK_START - exports.MEM_STACK_SIZE, exports.MEM_STACK_SIZE, exports.MEM_GROW_DOWN);
    exports.memRegions = [exports.memText, exports.memData, exports.memStack];
    exports.currentState = new cpuState();
};
exports.initializeMem = initializeMem;
/* INSTRUCTION COUNT ADD */
var instAddOne = function () {
    exports.INSTRUCTION_COUNT += 1;
};
exports.instAddOne = instAddOne;
var numInstSub = function () {
    exports.NUM_INST -= 1;
};
exports.numInstSub = numInstSub;
var changeRunBit = function () {
    exports.RUN_BIT = 0;
};
exports.changeRunBit = changeRunBit;
