import { instruction, Iinstruction, SymbolTableType } from '@utils/constants';
export interface simulatorOutputType {
    readonly PC: string;
    readonly registers: {
        [key: string]: string;
    };
    readonly dataSection: {
        [key: string]: string;
    } | Record<string, never>;
    readonly stackSection: {
        [key: string]: string;
    } | Record<string, never>;
}
export interface IBinaryData {
    readonly lineNumber: number;
    readonly data: string;
}
export interface IMapDetail {
    readonly key: number;
    readonly assembly: string;
    readonly binary: IBinaryData[];
}
export declare function parseInstr(buffer: string, index: number): instruction;
export declare function parseData(buffer: string, index: number): void;
export declare function printParseResult(INST_INFO: Iinstruction[], textSize: number, dataSize: number): void;
export declare function numToBits(num: number, pad?: number): string;
export declare function toHexAndPad(num: number, pad?: number): string;
export declare function symbolTableAddEntry(symbol: SymbolTableType): void;
export declare function log(printType: number, content: string): void;
export declare function isEmpty(value: string | null | undefined | object): boolean;
export declare function makeInput(inputFolderName: string, inputFileName: string): string[];
export declare function simulatorUnitTest(testCase: simulatorOutputType, output: simulatorOutputType): void;
export declare function parseSimulatorOutput(rawOutput: string): simulatorOutputType;
export declare function makeOutput(inputFolderName: string, inputFileName: string): string;
export declare function makeObjectFile(outputFolderPath: string, outputFileName: string, content: string[]): void;
export declare function makeMappingDetail(assemblyFile: string[], dataSeg: string[], textSeg: string[], mappingTable: number[][], output: string[]): IMapDetail[];
export declare function fromBinary(bits: string): number;
export declare function memRead(address: number): number;
export declare function memWrite(address: number, value: number): void;
export declare function memWriteHalf(address: number, value: number): void;
export declare function initMemory(): void;
export declare function initInstInfo(NUM_INST: number, INST_INFO: Iinstruction[]): void;
export declare function getInstInfo(pc: number): instruction;
export declare function mainProcess(INST_INFO: instruction[], cycles: number, CYCLES: simulatorOutputType[]): Promise<simulatorOutputType>;
export declare function cycle(): void;
export declare function dumpMemory(): string;
export declare function mdump(start: number, stop: number): string;
export declare function rdump(): string;
export declare function running(num_cycles: number, CYCLES: simulatorOutputType[]): void;
